package uaa.feral.context;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import repast.simphony.context.Context;
import repast.simphony.context.space.gis.GeographyFactoryFinder;
import repast.simphony.context.space.graph.NetworkFactoryFinder;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.gis.Geography;
import repast.simphony.space.gis.GeographyParameters;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.RandomGridAdder;
import repast.simphony.space.grid.StickyBorders;
import uaa.feral.physical.HydrologicalSetting;
import uaa.feral.physical.HydrologicalUnit;
import uaa.feral.physical.network.HydrologyEdge;
import uaa.feral.social.Community;
import uaa.feral.social.Organization;
import uaa.feral.social.network.RelationshipEdge;
import uaa.feral.util.data.ScenarioData;
import uaa.feral.util.loader.ScenarioXMLFileLoader;

/**
 * Class to load data and build the simulation.
 * This class is the first object called.
 * @author 
 */
public class FeralContextBuilder implements ContextBuilder{
	
	public static String PathToData="../FERAL/FERAL_Data/";  //path to main data folder
	public String scenarioData="";  //scenario xml data file reference used to setup the scenario
	private Network<RelationshipEdge> populationNetwork;  //social network of communities   Note:  not currently used
	private Network<HydrologyEdge> hydrologyNetwork;  //hydrological network   Note:  not currently used
	private Geography geography;  //geography the simulation belongs in
	private OutputContext outputContext;
	
	/**
	 * Builds and returns a context. Building a context consists of filling it with
	 * agents, instantiating projections, and any other simulation setup. 
	 * 
	 * @param context
	 * @return the built context.
	 */
	public Context build(Context context) {
		
		 populationNetwork=NetworkFactoryFinder.createNetworkFactory(null).
	      	createNetwork("Population", context, true);
		 
		  hydrologyNetwork=NetworkFactoryFinder.createNetworkFactory(null).
	      	createNetwork("Hydrology", context, true);
		
		  GeographyParameters geoParams = new GeographyParameters();
		  geography = GeographyFactoryFinder.createGeographyFactory(null)
          	.createGeography("Geography", context, geoParams);
		
		  // Get general simulation parameters
		  Parameters p = RunEnvironment.getInstance().getParameters();
		 
		  PathToData=p.getValueAsString("dataDirectory");
		  scenarioData=p.getValueAsString("scenarioData");
		
		  this.outputContext = new OutputContext("output","output");
		  context.addSubContext(outputContext);
		  
		  GridFactoryFinder.createGridFactory(null).createGrid("Simple Grid",
					context, GridBuilderParameters.singleOccupancy2D(new RandomGridAdder<RelationshipEdge>(),
									new StickyBorders(), 1000, 1000));
		  
		  //build from files
		  buildFromFiles(context);
		  
		  return context;
	}
	
	/**
	 * Method to get input files referenced by a scenario file
	 * @param context the master context
	 * @param fileData the path to the scenario file
	 */
	public void buildFromFiles(Context context){
		ScenarioXMLFileLoader sxf=ScenarioXMLFileLoader.SetupScenarioFileData(PathToData.concat(scenarioData));
		Iterator<ScenarioData> ic = sxf.getPopulations().iterator();
		ScenarioBuilder sc = new ScenarioBuilder(populationNetwork,outputContext);
		
		List<HydrologicalUnit>hydroUnits = new ArrayList<HydrologicalUnit>();
		while(ic.hasNext()){
			ScenarioData sd = ic.next();
			
			if(sd.isMiningScenario()){
				List<Organization> os = sc.buildMine(sd,sd.getMiningWaterUseModel());
				Iterator<Organization> icg = os.iterator();
				while(icg.hasNext()){
					Organization o = icg.next();
					context.addSubContext(o);
					context.add(o);
				}
			}
			
			if(sd.getCommunityFile()!=null){   //build a population and water use model
				List<Community>communities=sc.buildPopulation(sd,geography);
				Iterator<Community> ico = communities.iterator();
				while(ico.hasNext()) {
					Community c = ico.next();
					context.addSubContext(c);
					context.add(c);
				}
			}
			
			if(sd.getStreamFlowFile()!=null || sd.getHydrologicalFile()!=null) {    //any scenario with interactive hydrology
				HydrologicalSetting hs=sc.buildHydrology(sd,sd.getHydrologyModel(),geography);
				context.addSubContext(hs);
				context.add(hs);
			}
		}
	}
}
