package uaa.feral.context;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.PriorityQueue;

import repast.simphony.context.DefaultContext;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.space.graph.Network;
import simphony.util.messages.MessageCenter;
import uaa.feral.physical.HydrologicalUnit;
import uaa.feral.social.Person;
import uaa.feral.social.network.RelationshipEdge;
import uaa.feral.util.FilterDataOutput;

/**
 * Object to contain data to output and to output that data
 * @author 
 */
public class OutputContext extends DefaultContext<Object> {
	
	public static String Delimiter=",";  //delimit the data by this
	private static int FileNumber=1;  //for outputs within one run set
	private List<String> socialOutput=new ArrayList<String>();  //social related data
	private List<String> physicalOutput=new ArrayList<String>();  //physical object data
	
	private String socialTitleLine;  //title line on top of the social output file
	private String physicalTitleLine; //title line on top of the physical output file
	String physicalOutputFile="../FERAL/FERAL_Data/Output/PhysicalDataOutput";  //location and name of hydro base file
	String socialOutputFile="../FERAL/FERAL_Data/Output/SocialDataOutput";  //location and name of person base file
	String ending=".csv";  //type of file to output
	private FilterDataOutput fdo;  //object to print data output and to filter any possible data prior to output
	private int yes;
	private int no;
	/**
	 * Constructor for outputting data
	 */
	public OutputContext(){
		super();
		this.clear();
		socialOutput.clear();
	//	openOld();
	}
	
	/**
	 * Default constructor for outputting data
	 * @param output an output name
	 * @param outputContext an output context
	 */
	public OutputContext(String output, String outputContext){
		super(output,outputContext);
		this.clear();
		physicalOutput.clear();
		openOld();
	}
	
	/**
	 * Method to open an old file to merge with previous runs 
	 */
	public void openOld(){
		this.fdo=new FilterDataOutput();
		try{
			List<String> oldOutput=fdo.readTheseLines(this.socialOutputFile);
			if(oldOutput!=null)
				socialOutput=oldOutput;
		}
		catch(Exception e){
			
		}
		try{
			List<String> oldOutput=fdo.readTheseLines(this.physicalOutputFile);
			if(oldOutput!=null)
				physicalOutput=oldOutput;
		}
		catch(Exception e){
			
		}
	}
	
	public void recordAgentOutput(Object o,int tick){
		String[] getters=this.socialTitleLine.split(",");
		String time=Integer.toString(tick);
		String results=time;
		for(int i=0; i < getters.length;i++){
			String get = getters[i];
			if(get.equalsIgnoreCase("Tick"))
				continue;
			Method m=null;
			try {
				m = o.getClass().getDeclaredMethod("get"+get);
			} catch (SecurityException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			Object result = null;
			try {
				result = m.invoke(o);
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			results+=","+result.toString();
		}
		socialOutput.add(results);
	}
	
	public void recordRelationshipEdge(List<Double>results){
		if(this.getProjection("Population")!=null){
			Network network = ((Network)this.getProjection("Population"));
			Iterator<RelationshipEdge>ri=network.getEdges().iterator();
			int i=0;
			while(i<network.size()){
				String output="";
				for(int j=0; j < results.size(); j++){
					Double result = results.get(j);
					String value=Double.toString(result);
					if(i==results.size()-1)
						output+=value;
					else
						output+=value+Delimiter;
					socialOutput.add(output);
				}
				i++;
			}
		}
	}
	
	/**
	 * Method to record water use output for a given time tick
	 */
	public synchronized void recordWaterUseOutput(){
		List<Person>removePerson=new ArrayList<Person>();
		yes=0;
		no=0;
		try{
			double tick=RunEnvironment.getInstance().getInstance().getCurrentSchedule().getTickCount();
			String tickC=Double.toString(tick);
			if(this.getObjects(Person.class)!=null){
				if(socialTitleLine==null)
					socialTitleLine="Tick,Id,Age,QuantityBelief,QuantityChangeBelief,QualityBelief," +
							"QualityChangegBelief,OtherSourceQuantityBelief,OtherSourceQuantityChangeBelief,SourceVisited";
				Iterator ip=this.getObjects(Person.class).iterator();
				while(ip.hasNext()){
					Person p = (Person)ip.next();
					if(!p.isAlive()) {
						removePerson.add(p);
						continue;
					}
					if(p.isAccuratePerception())
						yes++;
					else
						no++;
					String age=Integer.toString(p.getAge());
					String id=Long.toString(p.getId());
					String quantityBelief=Double.toString(p.getQuantityBelief());
					String quantityChangeBelief=Double.toString(p.getQuantityChangeBelief());
					String qualityBelief=Double.toString(p.getQualityBelief());
					String qualityChangeBelief=Double.toString(p.getQualityChangeBelief());
					String sourceVisited=Integer.toString(p.getSourceVisited());
					String otherSourceQuantityBelief=Double.toString(p.getOtherSourceQuantityBelief());
					String otherSourceQChangeBelief=Double.toString(p.getOtherSourceQuantityChangeBelief());
					String output=tickC+Delimiter+id+Delimiter+age+Delimiter+quantityBelief+
						Delimiter+quantityChangeBelief+Delimiter+qualityBelief+Delimiter+qualityChangeBelief
						+Delimiter+otherSourceQuantityBelief+Delimiter+otherSourceQChangeBelief+Delimiter+sourceVisited;
				  //System.out.println(output);
					socialOutput.add(output);
				}
			}
		
			if(this.getObjects(HydrologicalUnit.class)!=null){
				if(physicalTitleLine==null)
					physicalTitleLine="Tick,Id,Volume,Discharge,Quality,TimesVisited,DistanceToTown," +
							"AccuratePerception,InaccuratePerception";
			
				Iterator hi=this.getObjects(HydrologicalUnit.class).iterator();
				while(hi.hasNext()){
					HydrologicalUnit hu =(HydrologicalUnit)hi.next();
					String id = Integer.toString(hu.getIdentification());
					String volume=Double.toString(hu.getVolume());
					String discharge=Double.toString(hu.getDischarge());
					String quality=Double.toString(hu.getQuality());
					String timesVisited=Integer.toString(hu.getTimesVisited());
					String distanceToTown=Double.toString(hu.getDistanceToTown());
					String output=tickC+Delimiter+id+Delimiter+volume+Delimiter+discharge+Delimiter+
					quality+Delimiter+timesVisited+Delimiter+distanceToTown+Delimiter+yes+Delimiter+no;
   //				System.out.println(output);
					physicalOutput.add(output);
				}
			}
		}
		catch(Exception e){
			MessageCenter.getMessageCenter(OutputContext.class).error("Problems with output context" +
					"output recording", e);
		}
		if(!removePerson.isEmpty())
			this.removeAll(removePerson);
	}
	
	public PriorityQueue<HydrologicalUnit> sortHydrology(Iterator iu){
		int size=this.getObjects(HydrologicalUnit.class).size();
		PriorityQueue<HydrologicalUnit> unsettled = new PriorityQueue<HydrologicalUnit>( size,
				new Comparator<HydrologicalUnit>() {
				public int compare(HydrologicalUnit o1, HydrologicalUnit o2) {
					double distance=o1.getDistanceToTown();
					double distance2=o2.getDistanceToTown();	
					return distance < distance2 ? -1 : distance==distance2 ? 0 : 1;
				}
			}); 
		while(iu.hasNext()){
			unsettled.add((HydrologicalUnit)iu.next());
		}
		return unsettled;
	}
	/**
	 * Method to compile all the data and print
	 */
	public void compileOutput(){
		
		if(!socialOutput.isEmpty()) {
			try{
				fdo.printThisFile(socialOutput, this.socialTitleLine, socialOutputFile+FileNumber+ending);
			}
			catch(Exception e){
			}
		}
		if(!physicalOutput.isEmpty()) {
			try{
				fdo.printThisFile(physicalOutput, this.physicalTitleLine, physicalOutputFile+FileNumber+ending);
			}
			catch(Exception e){
			
			}
		}
		FileNumber++;
	}

	public List<String> getSocialOutput() {
		return socialOutput;
	}

	public void setSocialOutput(List<String> socialOutput) {
		this.socialOutput = socialOutput;
	}

	public List<String> getPhysicalOutput() {
		return physicalOutput;
	}

	public void setPhysicalOutput(List<String> physicalOutput) {
		this.physicalOutput = physicalOutput;
	}

	public String getSocialTitleLine() {
		return socialTitleLine;
	}

	public void setSocialTitleLine(String socialTitleLine) {
		this.socialTitleLine = socialTitleLine;
	}

	public String getPhysicalTitleLine() {
		return physicalTitleLine;
	}

	public void setPhysicalTitleLine(String physicalTitleLine) {
		this.physicalTitleLine = physicalTitleLine;
	}

	public String getPhysicalOutputFile() {
		return physicalOutputFile;
	}

	public void setPhysicalOutputFile(String physicalOutputFile) {
		this.physicalOutputFile = physicalOutputFile;
	}

	public String getSocialOutputFile() {
		return socialOutputFile;
	}

	public void setSocialOutputFile(String socialOutputFile) {
		this.socialOutputFile = socialOutputFile;
	}
}
