package uaa.feral.physical.behaviors;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Map;

import org.apache.commons.lang.SystemUtils;

import repast.simphony.data.analysis.PluginOutputStream;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.IAction;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import simphony.util.messages.MessageCenter;

/**
 * Wrapper for Ground Water Model (e.g., MODFLOW and MT3DMS)
 * @author 
 */
public class GroundWaterModelWrapper extends EvolveHydrology{

	/**
	 * Default Constructor
	 */
	public GroundWaterModelWrapper(){
		super();
		this.name="GroundWaterModelWrapper";
	}
	
	private class RunExtra implements IAction{
		String argument;
		public RunExtra(String argument) {
			this.argument=argument;
		}
		public void execute() {
			runAdditionalProcess(argument);
		}	
	}
	
	public void setDischargeInfo(String river, int month,
			Map<Integer, double[]> sample) {
		// TODO Auto-generated method stub	
	}
	
	public void runModel(){
		if(this.count==0) {
			this.setupModelData();
		}
		
		ISchedule schedule1 = RunEnvironment.getInstance().getCurrentSchedule();
			schedule1.schedule(ScheduleParameters.createOneTime(RunEnvironment.getInstance().getCurrentSchedule().getTickCount(),2),new RunExtra(
			this.hmd.getNameOfParticleTransportFile()));
		
		ISchedule schedule2 = RunEnvironment.getInstance().getCurrentSchedule();
			schedule2.schedule(ScheduleParameters.createOneTime(RunEnvironment.getInstance().getCurrentSchedule().getTickCount(),1),new RunExtra(this.hmd.getAdditionalArgument()));
		
		count++;
	}
	
	/**
	 * Method to execute model processes
	 * @param commandArgument the command argument for the process
	 */
	private void runAdditionalProcess(String commandArgument){
		Process process=null;
		try {
			PluginOutputStream pos;
			if (SystemUtils.IS_OS_WINDOWS){
				
					process = Runtime.getRuntime().exec(commandArgument);
					try {
						process.waitFor();
					} catch (InterruptedException e) {
						MessageCenter.getMessageCenter(GroundWaterModelWrapper.class).error("GWM: Error, could not wait for execution to complete" +
								" The execution of GWM may not have waited or finished properly.", e);
					}
			}
			else{
				ProcessBuilder builder = new ProcessBuilder(commandArgument).redirectErrorStream(true);
				process = builder.start();
				try {
					process.waitFor();
				} catch (InterruptedException e) {
					MessageCenter.getMessageCenter(GroundWaterModelWrapper.class).error("GWM: Error, could not wait for execution to complete" +
							" The execution of GWM may not have waited or finished properly.", e);
				}
				pos = new PluginOutputStream(process.getInputStream());
				pos.start();
				OutputStream s=process.getOutputStream();
				s.flush();
			}
			process.getInputStream().close();
		}
	
		catch(IOException e){
			MessageCenter.getMessageCenter(GroundWaterModelWrapper.class).error("GWM: Error, preparing GWM execution." +
					" The execution of GWM may not occur properly.", e);
			
			PluginOutputStream pos2 = new PluginOutputStream(process.getErrorStream());
			pos2.start();
		}
//		process.destroy();
	}
}
