package uaa.feral.social;

import gov.nasa.worldwind.layers.IconLayer;

import java.util.Iterator;
import java.util.List;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.IAction;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.visualization.IDisplay;
import repast.simphony.visualization.gis3D.DisplayGIS3D;

import com.vividsolutions.jts.geom.Point;

/**
 * Object used to represent organizations, such as companies or government entities
 * @author 
 */
public class Organization extends Community {
	
	public Organization(){
		 super();
	}
	
	/**
	 * Class to run scheduled water use model behaviors
	 * @author Mark Altaweel
	 *
	 */
	private class RunWaterUse implements IAction{
		public void execute(){
			runModels();
		}
	}
	
	
	/**
	 * Default constructor used
	 * @param id the id of the organization
	 * @param location the geometric location
	 */
	public Organization(String id,Point location){
		 super(id,location);
	}
	
	/**
	 * Method to initialize parameters, behaviors, and call visualization initialization
	 */
	@ScheduledMethod(start = 0, pick=ScheduledMethod.ALL, priority = Double.MAX_VALUE)
	public void init(){
		 
		  ISchedule schedule2 = repast.simphony.engine.environment.RunEnvironment.getInstance()
		      	.getCurrentSchedule();
		  schedule2.schedule(ScheduleParameters.createOneTime(1, 0),new RunWaterUse());
		  
		  if(!RunEnvironment.getInstance().isBatch())
			  initV();	 
	}
	
	/**
	 * Method to initialize mine visualization
	 */
	private void initV(){
		List<IDisplay> displays=RunState.getInstance().getGUIRegistry().getDisplays();
		Iterator<IDisplay> ic = displays.iterator();
		DisplayGIS3D gis=null;
		while(ic.hasNext()){
			IDisplay display = ic.next();
			if(display instanceof DisplayGIS3D)
				gis=(DisplayGIS3D)display;
		}	
		IconLayer layer = new IconLayer();
		layer.addIcon(this.getIcon());
		gis.getWwglCanvas().getModel().getLayers().add(layer);
	}
	
	public void runModels(){
		waterUseModel.runModel();
	}
}
