package uaa.feral.social;

import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.geom.Position;
import gov.nasa.worldwind.layers.IconLayer;
import gov.nasa.worldwind.render.Renderable;
import gov.nasa.worldwind.render.UserFacingIcon;

import java.awt.Dimension;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.space.gis.Geography;
import repast.simphony.visualization.IDisplay;
import repast.simphony.visualization.gis3D.AbstractDisplayLayerGIS3D;
import repast.simphony.visualization.gis3D.DisplayGIS3D;
import repast.simphony.visualization.gis3D.SphereShape;

import com.vividsolutions.jts.geom.Point;

import uaa.feral.social.demographics.Sex;
import uaa.feral.social.network.AgentType;
import uaa.feral.social.network.Status;
import uaa.feral.social.waterUse.WaterUseModel;
import uaa.feral.style.WWSphereFERAL;

/**
 * Class for the person agent in the simulation
 * @author 
 */
public class Person {

	private Point location; //geometry of home location
	private SphereShape shape;  //shape used to govern movements of people and changes of person location that are displayed
	private boolean alive;
	private int age;
	private Sex sex;
	public static int Id=0;
	private int id;
	private WaterUseModel waterUseModel;  //model for water use
	private Community community;  //person's community
	private double quantityBelief;  //belief of water quantity
	private double quantityChangeBelief;  //change belief value
	private double qualityBelief;
	private double qualityChangeBelief;
	private UserFacingIcon icon;  //person icon
	private double sourcePreference;  //prefernce for a water source
	private int sourceVisited;  //current source visited by agent
	private double otherSourceQuantityBelief;
	private double otherSourceQuantityChangeBelief;
	private double otherSourceQualityChangeBelief;
	private double otherSourceQualityBelief;
	private double motivation; //motivation to accomplish tasks
	private boolean accuratePerception;
	private double waterConsumed;  //water consumed at given time
	private AgentType agentType;  //type of agent
	private Status status;  //status of the agent in the eyes of other agents
	private double importance; //how much an agent affects operations of social behavior
	private double decisionCost; //variable for cost of decision making
	private double x;  //arbitrary x location
	private double y; //arbitrary y location
	private int comply=0; //agree on something or not
	
	public double getDecisionCost() {
		return decisionCost;
	}

	public void setDecisionCost(double decisionCost) {
		this.decisionCost = decisionCost;
	}

	public double getImportance() {
		return importance;
	}

	public void setImportance(double importance) {
		this.importance = importance;
	}

	/**
	 * Default constructor
	 */
	public Person(){
		
	}

	/**
	 * Main constructor used for a person
	 * @param community the community the person belongs to
	 * @param age initial age
	 * @param sex the sex
	 * @param location person's initial location
	 */
	public Person(Community community, int age,Sex sex,Point location){
		super();
		this.age=age;
		this.alive=true;
		this.sex=sex;
		this.id=Id;
		this.location=location;
		this.community=community;
		Id++;
	}
	
	/**
	 * Method to move an agent (in animation mode) to a location
	 * @param newPoint
	 */
	public void moveTo(Point newPoint){
//		if(newPoint.getCentroid().getX()==community.getLocation().getCentroid().getX() &&
//			newPoint.getCentroid().getY()==community.getLocation().getCentroid().getY())
//			return;
		if(shape==null && !RunEnvironment.getInstance().isBatch())
			initV(newPoint);
		Context context = RunState.getInstance().getMasterContext();  
		Geography geography = (Geography)context.getProjection("Geography");
		this.location=newPoint;
	//	geography.move(this,location);
		if(this.icon!=null){
//			this.icon.getPosition().getLatLon().getLatitude().=newPoint.getCentroid().getY();
//			this.icon.getPosition().getLatLon().getLongitude().degrees=newPoint.getCentroid().getX();
//			this.icon.move(this.icon.getPosition());
			this.icon.moveTo( new Position(Angle.fromDegreesLatitude(newPoint.getCoordinate().y), 
				Angle.fromDegreesLongitude(newPoint.getCoordinate().x), 0.0));
		}
	}
	
	public WaterUseModel getWaterUseModel() {
		return waterUseModel;
	}

	public UserFacingIcon getIcon() {
		return icon;
	}

	public void setIcon(UserFacingIcon icon) {
		this.icon = icon;
	}

	public void setWaterUseModel(WaterUseModel waterUseModel) {
		this.waterUseModel = waterUseModel;
	}
	
	/**
	 * Method for visualization of a person, used in animation mode
	 */
	public void initV(Point newPoint){
		List<IDisplay> displays=RunState.getInstance().getGUIRegistry().getDisplays();
		Iterator<IDisplay> ic = displays.iterator();
		DisplayGIS3D gis=null;
		while(ic.hasNext()){
			IDisplay display = ic.next();
			if(display instanceof DisplayGIS3D)
				gis=(DisplayGIS3D)display;
		}
		if(gis==null)
			return;
		Map<Class, AbstractDisplayLayerGIS3D> objs=gis.getClassStyleMap();
		AbstractDisplayLayerGIS3D layer = objs.get(Community.class);
		Collection<Renderable> renderables=(Collection<Renderable>)layer.getRenderables();
		SphereShape pS=new WWSphereFERAL(location,15, "person");
//		layer.getShapeToObjectMap().put(pS, this);
		this.shape=pS;
//		renderables.add(pS );
//		gis.getWwglCanvas().redraw();
		Context context = RunState.getInstance().getMasterContext();  
		Geography geography = (Geography)context.getProjection("Geography");
//		geography.move(this, location);
		
        IconLayer layer2 = new IconLayer();
//      icon = new UserFacingIcon("src/images/32x32-icon-nasa.png",
        UserFacingIcon icon = new UserFacingIcon("../FERAL/img/People.png",
              new Position(Angle.fromDegreesLatitude(newPoint.getCentroid().getY()),
            		  Angle.fromDegrees(newPoint.getCentroid().getX()), 0));
        icon.setSize(new Dimension(14, 14));
        this.icon=icon;
        layer2.addIcon(icon);
		gis.getWwglCanvas().getModel().getLayers().add(layer2);
	}
	
	/**
	 * Method for launching agent water use model (not currently used, as the behavior is done through the community)
	 */
//	@ScheduledMethod(start = 0, interval = 1, pick=ScheduledMethod.ALL, priority = 0)
	public void runWaterUseMode(){
//		waterUseModel.runModel();
	}
	
	public int getAge() {
		return age;
	}
	
	public void setAge(int age) {
		this.age = age;
	}
	public Sex getSex() {
		return sex;
	}
	public void setSex(Sex sex) {
		this.sex = sex;
	}
	
	public boolean isAlive() {
		return alive;
	}

	public void setAlive(boolean alive) {
		this.alive = alive;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public Community getCommunity() {
		return community;
	}

	public void setCommunity(Community community) {
		this.community = community;
	}

	public Renderable getShape() {
		return shape;
	}

	public double getQuantityBelief() {
		return quantityBelief;
	}

	public void setQuantityBelief(double quantityBelief) {
		this.quantityBelief = quantityBelief;
	}

	public double getQuantityChangeBelief() {
		return quantityChangeBelief;	
	}

	public void setQuantityChangeBelief(double quantityChangeBelief) {
		this.quantityChangeBelief = quantityChangeBelief;
	}

	public double getSourcePreference() {
		return sourcePreference;
	}

	public void setSourcePreference(double sourcePreference) {
		this.sourcePreference = sourcePreference;
	}

	public double getQualityBelief() {
		return qualityBelief;
	}

	public void setQualityBelief(double qualityBelief) {
		this.qualityBelief = qualityBelief;
	}

	public double getQualityChangeBelief() {
		return qualityChangeBelief;
	}

	public void setQualityChangeBelief(double qualityChangeBelief) {
		this.qualityChangeBelief = qualityChangeBelief;
	}

	public double getOtherSourceQuantityBelief() {
		return otherSourceQuantityBelief;
	}

	public void setOtherSourceQuantityBelief(double otherSourceQuantityBelief) {
		this.otherSourceQuantityBelief = otherSourceQuantityBelief;
	}

	public double getOtherSourceQuantityChangeBelief() {
		return otherSourceQuantityChangeBelief;
	}

	public void setOtherSourceQuantityChangeBelief(
			double otherSourceQuantityChangeBelief) {
		this.otherSourceQuantityChangeBelief = otherSourceQuantityChangeBelief;
	}

	public double getOtherSourceQualityChangeBelief() {
		return otherSourceQualityChangeBelief;
	}

	public void setOtherSourceQualityChangeBelief(
			double otherSourceQualityChangeBelief) {
		this.otherSourceQualityChangeBelief = otherSourceQualityChangeBelief;
	}

	public double getOtherSourceQualityBelief() {
		return otherSourceQualityBelief;
	}

	public void setOtherSourceQualityBelief(double otherSourceQualityBelief) {
		this.otherSourceQualityBelief = otherSourceQualityBelief;
	}

	public double getX() {
		return x;
	}

	public double getY() {
		return y;
	}

	public boolean isAccuratePerception() {
		return accuratePerception;
	}

	public void setAccuratePerception(boolean accuratePerception) {
		this.accuratePerception = accuratePerception;
	}

	public double getWaterConsumed() {
		return waterConsumed;
	}

	public void setWaterConsumed(double waterConsumed) {
		this.waterConsumed = waterConsumed;
	}

	public int getSourceVisited() {
		return sourceVisited;
	}

	public void setSourceVisited(int sourceVisited) {
		this.sourceVisited = sourceVisited;
	}
	
	public Point getLocation() {
		return location;
	}

	public void setLocation(Point location) {
		this.location = location;
	}

	public AgentType getAgentType() {
		return agentType;
	}

	public void setAgentType(AgentType agentType) {
		this.agentType = agentType;
	}

	public Status getStatus() {
		return status;
	}

	public void setStatus(Status status) {
		this.status = status;
	}

	public void setX(double x) {
		this.x = x;
	}

	public void setY(double y) {
		this.y = y;
	}

	public double getMotivation() {
		return motivation;
	}

	public void setMotivation(double motivation) {
		this.motivation = motivation;
	}

	public int getComply() {
		return comply;
	}

	public void setComply(int comply) {
		this.comply = comply;
	}
}
