package uaa.feral.social.data;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import repast.simphony.random.RandomHelper;
import simphony.util.messages.MessageCenter;
import uaa.feral.social.network.AgentType;
import uaa.feral.util.loader.ScenarioXMLFileLoader;

import cern.jet.random.Normal;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

public class AppliedSocialModelData implements SocialModelData {
	private Map<String,Double> ageWeightedDecisions;
	private Map<String,Double> sourceWeightedPerceptions;
	private Map<Object,Double> statusWeightedDecisions;
	private double municipalConsumptionRate; //consumption per person
	private double nonMunicipalConsumptionRate; //consumption per person
	private boolean municipal=false;
	private boolean largeSim=false;
	private boolean degradeQuality=false;
	private double tolerableDistanceToSource;
	
	public static String OutputDirectory= "../FERAL/FERAL_Data/SocialData/ModelData/";
	
	public AppliedSocialModelData(){
		
	}
	
	public AppliedSocialModelData setupModelData(String file){
		XStream xstream = new XStream(new DomDriver());
		xstream.setClassLoader(AppliedSocialModelData.class.getClassLoader());
		xstream.alias("AppliedSocialModelData",AppliedSocialModelData.class);
		xstream.aliasType("AppliedSocialModelData", AppliedSocialModelData.class);
		AppliedSocialModelData sdmd=this;
		try {
			sdmd=(AppliedSocialModelData)xstream.fromXML(new FileReader(file));
			
		} catch (Exception e) {
			MessageCenter.getMessageCenter(AppliedSocialModelData.class).error("Problems with reading" +
					"input XML file for Social Distancing Data", e);
			e.printStackTrace();	

		}
		return sdmd;
	}
	
	public static void main(String[] args) throws FileNotFoundException, IOException{
		XStream xstream = new XStream(new DomDriver());
		xstream.alias("AppliedSocialModelData",uaa.feral.social.data.AppliedSocialModelData.class);
		
		AppliedSocialModelData sdmd = new AppliedSocialModelData();
		 
		Map<Object,Double> statusDecisions=new HashMap<Object,Double>();
		statusDecisions.put(AgentType.Beta, -1.0);
		statusDecisions.put(AgentType.Alpha, 1.0);
		statusDecisions.put(AgentType.Gamma, -1.0);
		statusDecisions.put("affectiveDependence1",2d);
		statusDecisions.put("affectiveDependence1E",8d);
		statusDecisions.put("affectiveDependence2", 2d);
		statusDecisions.put("affectiveDependence2E", 8d);
		statusDecisions.put("affectiveDependence3",8d);
		statusDecisions.put("affectiveDependence3E",2d);
		statusDecisions.put("normativeDependence1",8d);
		statusDecisions.put("normativeDependence1E",2d);
		statusDecisions.put("normativeDependence2", 2d);
		statusDecisions.put("normativeDependence2E", 8d);
		statusDecisions.put("normativeDependence3", 2d);
		statusDecisions.put("normativeDependence3E", 8d);
		
		sdmd.setStatusWeightedDecisions(statusDecisions);
		
		xstream.setMode(XStream.XPATH_RELATIVE_REFERENCES);
		xstream.aliasType("AppliedSocialModelData", AppliedSocialModelData.class);
		String xml=xstream.toXML(sdmd);
		String fileName= "AppliedSocialModelData2.xml";
		
		String filename= OutputDirectory+fileName;
		ScenarioXMLFileLoader.Output(xml,filename);
	}

	public Map<String, Double> getAgeWeightedDecisions() {
		return ageWeightedDecisions;
	}

	public void setAgeWeightedDecisions(Map<String, Double> waterDecisionAges) {
		this.ageWeightedDecisions = waterDecisionAges;
	}

	public boolean isMunicipal() {
		return municipal;
	}

	public void setMunicipal(boolean municipal) {
		this.municipal = municipal;
	}

	public boolean isLargeSim() {
		return largeSim;
	}

	public void setLargeSim(boolean largeSim) {
		this.largeSim = largeSim;
	}

	public boolean isDegradeQuality() {
		return degradeQuality;
	}

	public void setDegradeQuality(boolean degradeQuality) {
		this.degradeQuality = degradeQuality;
	}

	public double getTolerableDistanceToSource() {
		return tolerableDistanceToSource;
	}

	public void setTolerableDistanceToSource(double tolerableDistanceToSource) {
		this.tolerableDistanceToSource = tolerableDistanceToSource;
	}

	public Map<String, Double> getSourceWeightedPerceptions() {
		return sourceWeightedPerceptions;
	}

	public void setSourceWeightedPerceptions(
			Map<String, Double> sourceWeightedPerceptions) {
		this.sourceWeightedPerceptions = sourceWeightedPerceptions;
	}

	public double getMunicipalConsumptionRate() {
		return municipalConsumptionRate;
	}

	public void setMunicipalConsumptionRate(double municipalConsumptionRate) {
		this.municipalConsumptionRate = municipalConsumptionRate;
	}

	public double getNonMunicipalConsumptionRate() {
		return nonMunicipalConsumptionRate;
	}

	public void setNonMunicipalConsumptionRate(double nonMunicipalConsumptionRate) {
		this.nonMunicipalConsumptionRate = nonMunicipalConsumptionRate;
	}

	public Map<Object,Double> getStatusWeightedDecisions() {
		return statusWeightedDecisions;
	}

	public void setStatusWeightedDecisions(Map<Object,Double> statusWeghtedDecisions) {
		this.statusWeightedDecisions = statusWeghtedDecisions;
	}
}
