package uaa.feral.social.waterUse;

import uaa.feral.social.Community;
import uaa.feral.social.Organization;
import uaa.feral.util.data.ScenarioData;

/**
 * Class for use in water use behaviors
 * @author 
 */
public abstract class WaterUseBehaviors implements WaterUseModel{

	protected boolean updatePreferences=true;
	protected boolean largeSimulation=false;
	protected boolean degradeQualityMode=false;
	protected ScenarioData scenarioData;
	
	public void runModel(){
		//this should be implemented in the model
	}
	
	/**
	 * Static method to create the desired model based on model's name. Community object is then set to the model
	 * @param model a model to create
	 * @param community the community to associate the model to
	 * @return a WaterUseModel object
	 */
	public static WaterUseModel CreateWaterUseModel(String model,ScenarioData sd,Community community){
		
		WaterUseModel waterUseModel=null;
		
		//create models to use
		if(model.equals("SocialDistancingModel")){
			waterUseModel=new SocialDistancing(community);
		}
		
		else if(model.equals("SocialDistancingModelModified")){
			waterUseModel=new SocialDistancingModified(community);
		}
		
		else if(model.equals("MiningWaterUse")){
			waterUseModel=new MiningWaterUse((Organization)community);
		}
		((WaterUseBehaviors)waterUseModel).setScenarioData(sd);
		
		return waterUseModel;
	}
	
	public void setupModelData(String file){
		
	}
	
	public boolean isUpdatePreferences() {
		return updatePreferences;
	}

	public void setUpdatePreferences(boolean updatePreferences) {
		this.updatePreferences = updatePreferences;
	}
	
	public boolean isLargeSimulation() {
		return largeSimulation;
	}
	
	public void setLargeSimulation(boolean largeSimulation){
		this.largeSimulation=largeSimulation;
	}

	public boolean isDegradeQualityMode() {
		return degradeQualityMode;
	}

	public void setDegradeQualityMode(boolean degradeQualityMode) {
		this.degradeQualityMode = degradeQualityMode;
	}

	public ScenarioData getScenarioData() {
		return scenarioData;
	}

	public void setScenarioData(ScenarioData scenarioData) {
		this.scenarioData = scenarioData;
	}
}
