package uaa.feral.style;

import gov.nasa.worldwind.geom.LatLon;
import gov.nasa.worldwind.geom.Sphere;
import gov.nasa.worldwind.geom.Vec4;
import gov.nasa.worldwind.render.DrawContext;
import gov.nasa.worldwind.render.Material;
import gov.nasa.worldwind.util.Logging;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;
import javax.media.opengl.glu.GLUquadric;

import com.vividsolutions.jts.geom.Point;

import repast.simphony.visualization.gis3D.SphereShape;

/**
 * Class based on Sphere in Repast Visualization package, which uses WW.
 * @author 
 *
 */
public class WWSphereFERAL extends SphereShape {
	private Vec4 center;
	private double radius;
	private LatLon latlon;
	private boolean initialized = false;
	private String type;
	@SuppressWarnings("unused")
	private Object dc;

	private static int glListId;
	private static GLUquadric quadric;

	public WWSphereFERAL(Point point){
		super(point);
	}
	public WWSphereFERAL(Point point,double radius,String type){
		super(point);
		this.radius = radius;
		this.type=type;
	}

	public void initialize(DrawContext dc){
		if(type.equals("person"))  
			Material.YELLOW.apply(dc.getGL(), GL.GL_FRONT);
		else
			Material.RED.apply(dc.getGL(), GL.GL_FRONT);
		
		glListId = dc.getGL().glGenLists(1);
		quadric = dc.getGLU().gluNewQuadric();

		dc.getGLU().gluQuadricDrawStyle(quadric, GLU.GLU_FILL);
		dc.getGLU().gluQuadricNormals(quadric, GLU.GLU_SMOOTH);
		dc.getGLU().gluQuadricOrientation(quadric, GLU.GLU_OUTSIDE);
		dc.getGLU().gluQuadricTexture(quadric, false);

    int slices = 10;
    int stacks = 10;

		dc.getGL().glNewList(glListId, GL.GL_COMPILE);
		dc.getGLU().gluSphere(quadric, radius, slices, stacks);
		dc.getGL().glEndList();
		this.dc=dc;
		initialized = true;
	}

	@Override
	protected void doRender(DrawContext dc){
		center = dc.getGlobe().computePointFromPosition(latlon.getLatitude(), 
				latlon.getLongitude(), 250);
		
		double scale = radius * dc.getView().getEyePosition().getElevation() 
		                  / ZOOM_SCALE_FACTOR;
		
		dc.getView().pushReferenceCenter(dc, center);
		dc.getGL().glScaled(scale, scale, scale);
		dc.getGL().glCallList(glListId);
		dc.getView().popReferenceCenter(dc);
		this.dc=dc;
	}

	@Override
	public void render(DrawContext dc) {
		if (dc == null) {
			String msg = Logging.getMessage("nullValue.DrawContextIsNull");
			Logging.logger().severe(msg);
			throw new IllegalArgumentException(msg);
		}

		if (!initialized)
			initialize(dc);
		
////	center = dc.getSurfaceGeometry().getSurfacePoint(latlon.getLatitude(), latlon.getLongitude(), 1e4);

		center = dc.getGlobe().computePointFromPosition(latlon.getLatitude(), latlon.getLongitude(), 1e4);


		begin(dc);
		
		if(type.equals("person"))  
			Material.YELLOW.apply(dc.getGL(), GL.GL_FRONT);
		else
			Material.RED.apply(dc.getGL(), GL.GL_FRONT);
		
		dc.getView().pushReferenceCenter(dc, center);
		this.doRender(dc);
		dc.getView().popReferenceCenter(dc);
		
		end(dc);

	}

	public void setCenter(Vec4 center) {
		this.center = center;
	}

	public void setRadius(double radius) {
		this.radius = radius;
	}

	public void setLatlon(LatLon latlon) {
		this.latlon = latlon;
	}

	
	private static void begin(DrawContext dc){
		GL gl = dc.getGL();
		Vec4 cameraPosition = dc.getView().getEyePoint();

		gl.glPushAttrib(
				GL.GL_TEXTURE_BIT | GL.GL_ENABLE_BIT | GL.GL_CURRENT_BIT | GL.GL_LIGHTING_BIT | GL.GL_TRANSFORM_BIT);
		gl.glDisable(GL.GL_TEXTURE_2D);

		float[] lightPosition =
		{(float) (cameraPosition.x * 2), (float) (cameraPosition.y / 2), (float) (cameraPosition.z), 0.0f};
		float[] lightDiffuse = {1.0f, 1.0f, 1.0f, 1.0f};
		float[] lightAmbient = {1.0f, 1.0f, 1.0f, 1.0f};
		float[] lightSpecular = {1.0f, 1.0f, 1.0f, 1.0f};

		gl.glDisable(GL.GL_COLOR_MATERIAL);

		gl.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, lightPosition, 0);
		gl.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE, lightDiffuse, 0);
		gl.glLightfv(GL.GL_LIGHT1, GL.GL_AMBIENT, lightAmbient, 0);
		gl.glLightfv(GL.GL_LIGHT1, GL.GL_SPECULAR, lightSpecular, 0);

		gl.glDisable(GL.GL_LIGHT0);
		gl.glEnable(GL.GL_LIGHT1);
		gl.glEnable(GL.GL_LIGHTING);
		gl.glEnable(GL.GL_NORMALIZE);

		gl.glMatrixMode(GL.GL_MODELVIEW);
		gl.glPushMatrix();
	}

	private static void end(DrawContext dc) {
		GL gl = dc.getGL();

		gl.glMatrixMode(GL.GL_MODELVIEW);
		gl.glPopMatrix();

		gl.glDisable(GL.GL_LIGHT1);
		gl.glEnable(GL.GL_LIGHT0);
		gl.glDisable(GL.GL_LIGHTING);
		gl.glDisable(GL.GL_NORMALIZE);
		gl.glPopAttrib();
	}
}
