package uaa.feral.style;

import gov.nasa.worldwind.globes.Globe;
import gov.nasa.worldwind.geom.*;
import gov.nasa.worldwind.render.DrawContext;
import gov.nasa.worldwind.render.SurfaceCircle;
import gov.nasa.worldwind.util.Logging;

import java.awt.*;
import java.util.ConcurrentModificationException;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;
import javax.media.opengl.glu.GLUquadric;

/**
 * Class based on Surface Circle object in WW
 * @author 
 */
public class WWSurfaceCircleFERAL extends SurfaceCircle 
{
    private double radius;
	private DrawContext dc;
	private LatLon center;
	private int glListId;
	private GLUquadric quadric;
	private boolean initialized=false;

	public WWSurfaceCircleFERAL(Globe globe, LatLon center, double radius, int intervals)
    {
        super(globe, center, radius, intervals);
        this.radius=radius;
        this.center=center;
    }

    public WWSurfaceCircleFERAL(Globe globe, LatLon center, double radius, int intervals, Color interiorColor,
        Color borderColor)
    {
        super(globe, center, radius, intervals, interiorColor, borderColor);
        this.radius=radius;
        this.center=center;
        
    }

    public WWSurfaceCircleFERAL(Globe globe, LatLon center, double radius, int intervals, Color interiorColor,
        Color borderColor, Dimension textureSize)
    {
        super(globe, center,  radius, intervals, interiorColor, borderColor, textureSize);
        this.radius=radius;
        this.center=center;
    }

	
	public void initialize(DrawContext dc){
		
		glListId = dc.getGL().glGenLists(1);
		quadric = dc.getGLU().gluNewQuadric();

		dc.getGLU().gluQuadricDrawStyle(quadric, GLU.GLU_FILL);
		dc.getGLU().gluQuadricNormals(quadric, GLU.GLU_SMOOTH);
		dc.getGLU().gluQuadricOrientation(quadric, GLU.GLU_OUTSIDE);
		dc.getGLU().gluQuadricTexture(quadric, false);

        int slices = 10;
        int stacks = 10;

		dc.getGL().glNewList(glListId, GL.GL_COMPILE);
		dc.getGLU().gluSphere(quadric, radius, slices, stacks);
		dc.getGL().glEndList();
		this.dc=dc;
		initialized = true;
	}


	public void render(DrawContext dc){
		try
			{
			super.render(dc);
			}
		catch(ConcurrentModificationException e){
			return;
		}
		Vec4 centr = dc.getGlobe().computePointFromPosition(center.getLatitude(), 
				center.getLongitude(), 0);
		
		double scale = (radius*2) * (dc.getView().getEyePosition().getElevation() 
		                  / WWSphereFERAL.ZOOM_SCALE_FACTOR);
		this.setRadius(scale);
		dc.getView().pushReferenceCenter(dc, centr);
		dc.getGL().glScaled(scale, scale, scale);
		dc.getGL().glCallList(glListId);
		dc.getView().popReferenceCenter(dc);
		this.dc=dc;
	}

	
    public double getRadius()
    {
        return super.getMajorAxisLength();
    }

    public void setRadius(double radius)
    {
        if (radius <= 0)
        {
            String message = Logging.getMessage("Geom.RadiusInvalid", radius);
            Logging.logger().severe(message);
            throw new IllegalArgumentException(message);
        }

        super.setAxisLengths(radius, radius);
    }

	/**
	 * A separate rendering method for picking.
	 * 
	 * @param dc
	 */
	public void pickRender(DrawContext dc){
		
		dc.getGL().glMatrixMode(GL.GL_MODELVIEW);
		dc.getGL().glPushMatrix();
				
		this.render(dc);
		
		dc.getGL().glMatrixMode(GL.GL_MODELVIEW);
		dc.getGL().glPopMatrix();
	}
}
