package uaa.feral.util.data;

import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import simphony.util.messages.MessageCenter;
import uaa.feral.util.loader.ScenarioXMLFileLoader;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

public class FilterData {

	private List<String>variableNames;
	private String variableTitles;
	private String fileName;
	private Map<String,Double> lessThan;
	private Map<String,Double> greaterThan;
	private Map<String,Integer>remove;
	
	public List<String> getVariableNames() {
		return variableNames;
	}
	public void setVariableNames(List<String> variableNames) {
		this.variableNames = variableNames;
	}
	public String getFileName() {
		return fileName;
	}
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}
	
	public String getVariableTitles() {
		return variableTitles;
	}
	public void setVariableTitles(String variableTitles) {
		this.variableTitles = variableTitles;
	}
	
	/**
	 * Method to setup filter data from an xml file
	 * @param file the file to load
	 */
	public static FilterData SetupFilterData(String file){
		XStream xstream = new XStream(new DomDriver());
		xstream.setClassLoader(FilterData.class.getClassLoader());
		xstream.alias("FilterData",FilterData.class);
		xstream.aliasType("FilterData", FilterData.class);
		FilterData fd=null;
		try {
			fd=(FilterData)xstream.fromXML(new FileReader(file));
			
		} catch (Exception e) {
			MessageCenter.getMessageCenter(FilterData.class).error("Problems with reading" +
					"input XML file for filter data", e);
			e.printStackTrace();	

		}
		return fd;
	}
	
	public static void main(String[] args){
		FilterData fd = new FilterData();
		
		XStream xstream = new XStream(new DomDriver());
		xstream.alias("FilterData",uaa.feral.util.data.FilterData.class);
		xstream.setMode(XStream.XPATH_RELATIVE_REFERENCES);
		
		String fileName="../FERAL/FERAL_Data/Output/PersonDataOutput.csv";
		String removeVariables="Age";
		String removeVariables2="QuantityBelief";
		String variableTitles="Tick,Id,Age,QuantityBelief,QuantityChangeBelief";
		
		Map<String,Double> lessThan = new HashMap<String,Double>();
		Map<String,Double> greaterThan = new HashMap<String,Double>();
		Map<String,Integer> remove = new HashMap<String,Integer>();
		
		lessThan.put(removeVariables, 13d);
		greaterThan.put(removeVariables, 65d);
		remove.put(removeVariables, 1);
		
		lessThan.put(removeVariables2, 0d);
		greaterThan.put(removeVariables2, Double.MAX_VALUE);
		remove.put(removeVariables2, 0);
		
		List<String> variableNames = new ArrayList<String>();
		variableNames.add(removeVariables);
		variableNames.add(removeVariables2);
		
		fd.setVariableNames(variableNames);
		fd.setFileName(fileName);
		fd.setVariableTitles(variableTitles);
		fd.setLessThan(lessThan);
		fd.setGreaterThan(greaterThan);
		fd.setRemove(remove);
		
		String xml=xstream.toXML(fd);
		
		String file= "FilterOutputData.xml";
		String outputDirectory= "../FERAL/FERAL_Data/Output/";
		
		String filename= outputDirectory+file;
		ScenarioXMLFileLoader.Output(xml,filename);
		
	}
	public Map<String, Double> getLessThan() {
		return lessThan;
	}
	public void setLessThan(Map<String, Double> lessThan) {
		this.lessThan = lessThan;
	}
	public Map<String, Double> getGreaterThan() {
		return greaterThan;
	}
	public void setGreaterThan(Map<String, Double> greaterThan) {
		this.greaterThan = greaterThan;
	}
	public Map<String, Integer> getRemove() {
		return remove;
	}
	public void setRemove(Map<String, Integer> remove) {
		this.remove = remove;
	}
}
