package uaa.feral.util.loader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

/**
 * Loader to load csv files
 * @author 
 */

public class CSVLoader {

	/**
	 * Method used to call the loader for the csv data
	 * @param fName a file name
	 * @return a list of maps that have data from the csv file
	 */
    public static List<Map> loadCSVStatic(String fName){
      return (new CSVLoader()).loadCSV(fName);
    }
	
	/**
	 * Default constructor
	 */
	public CSVLoader() {
	}
	
	/**
	 * Loader method for loading a csv file with column headings and row data
	 * @param fName represents the file name as a string object
	 * @param data is a map containing the string key and the list contaning the data map of that key
	 * @return the data map that was inputed will be returned
	 */
	public List<Map> loadCSV(String fName) {	
		fName=verifyFile(fName);
		List<Map> dataList = new ArrayList<Map>();
		Map<String, String> localValue= new HashMap<String, String>();
		
		try {
			File file = new File(fName);
			 
			BufferedReader bufRdr  = new BufferedReader(new FileReader(file));
			String line = null;
			int row = 0;
			int col = 0;
			String[] keys=null;
//			read each line of file
			while((line = bufRdr.readLine()) != null)
			{
				StringTokenizer st = new StringTokenizer(line,",");
				int size=st.countTokens();
				
				String[] values=new String[size];
				while (st.hasMoreTokens())
				{
					if(row==0) {
						if(keys==null)
							keys=new String[size];
						
						keys[col]=st.nextToken();
					}
					
					else {
						values[col]=st.nextToken();
						localValue.put(keys[col], values[col]);
					}
					col++;
				}
				
				if(row>0 ) {
					if(!localValue.isEmpty())
						dataList.add(localValue);
					localValue=new HashMap<String,String>();
				}
				col=0;
				row++;
			}
			 
//			close the file
			bufRdr.close(); 
		}
		
		catch(IOException e) {
			if(!fName.contains("null"))
					e.printStackTrace();
		}
		
	
		return dataList;
	}
	
	/**
	 * Method to correct file path reference information
	 * @param fName the path name
	 * @return the corrected path name
	 */
	private String verifyFile(String fName){
		fName=fName.replaceAll("\t", "");
		fName=fName.replaceAll("\n", "");
		
		return fName;
	}
}
