package uaa.feral.util.loader;


import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureResults;
import org.geotools.data.FeatureSource;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.feature.AttributeType;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureType;
import org.geotools.feature.IllegalAttributeException;

import simphony.util.messages.MessageCenter;


/**
 * Shapefile loader to parse shape file data
 * @author 
 */

public class ShapeFileLoader{
	
	/**
	 * Default constructor
	 */
	public ShapeFileLoader(){
		
	}
	
	/**
	 * Method to load a shapefile using geotools functions
	 * @param f the file you want to load
	 * @return a list with maps that have specific cell values from the shapefile
	 */
	public static List<Map> ShapeFileLoader(File f){

		List<Map> map = new ArrayList<Map>();
		URL shapeURL = null;
	
		try{
			shapeURL = f.toURI().toURL();
			
			// get feature results
			ShapefileDataStore store = new ShapefileDataStore(shapeURL);
			AttributeType[] at=store.getSchema().getAttributeTypes();
			String name = store.getTypeNames()[0];
			FeatureSource source = store.getFeatureSource(name);
			FeatureResults fsShape = source.getFeatures();
			FeatureReader reader = fsShape.reader();
			while (reader.hasNext()) {
				Map<String, Object> data = new HashMap<String, Object>();
				Feature feature = reader.next();
				for(int i=0; i < at.length; i++) {
					data.put(at[i].getName(),feature.getAttribute(i));
				}
				map.add(data);
				data = new HashMap<String, Object>();
			}
			//	System.out.println(map);
			reader.close();
		}catch(MalformedURLException e){
			System.out.println("Cannot find shape file");
		}catch(IOException e){
			System.out.println("Features not found");
		}catch (NoSuchElementException e) {
			MessageCenter.getMessageCenter(ShapeFileLoader.class).error("Problems with finding" +
					"element data in the file", e);
		} catch (IllegalAttributeException e) {
			MessageCenter.getMessageCenter(ShapeFileLoader.class).error("Problems with the proper" +
					"attribute in the file", e);
		}

		return map;
	}
	

	/**
	 * Method to parse features into a map
	 * @param f the feature to parse
	 * @param at the attribute types to associate data with
	 * @return a map with the feature data
	 */
	public static Map ParseFeature(Feature f, AttributeType at[]){
		Map<String, Object> data = new HashMap<String, Object>();
		FeatureType ft=f.getFeatureType();
		for(int i=0; i < at.length; i++) {
			data.put(at[i].getName(),f.getAttribute(i));
		}		
		return data;
	}
}
