package uaa.feral.physical.behaviors;

import java.util.HashMap;
import java.util.Map;

import repast.simphony.random.RandomHelper;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;

import uaa.feral.physical.HydrologicalSetting;
import uaa.feral.physical.HydrologicalUnit;

/**
 * Class to test Remote Sensing Estimator.
 * @author 
 */
public class RemoteSensingEstimatorTest {

	/**
	 * The main argument to run test on the remote sensing estimator model
	 * @param args
	 */
	public static void main(String[] args){
		for(int i=0; i < 50; i++) {
			RemoteSensingEstimatorTest rset = new RemoteSensingEstimatorTest();
		
			RemoteSensingEstimator rse = new RemoteSensingEstimator("testing");
			HydrologicalUnit hu = rset.createHydrologicalUnit();
			HydrologicalSetting hs = new HydrologicalSetting("test");
			rse.setHydrologicalSetting(hs);
			rset.setupWidthData(rse,hu);
			hs.add(hu);
			rse.runModel(7);
		
			System.out.println("Hydrological Discharge: "+hu.getDischarge());
		}
	}
	
	/**
	 * Create a hydrological unit setup for test
	 * @return a hydrological unit
	 */
	public HydrologicalUnit createHydrologicalUnit(){
		HydrologicalUnit hu=null;
		Coordinate c = new Coordinate(165.5,64.7,0);
		Coordinate[] coords = {c};
		Point p = new Point(new CoordinateArraySequence(coords),new GeometryFactory());
		hu=new HydrologicalUnit(p,null);
		hu.setWaterWidth(135d);
		hu.setBankfullWidth(160d);
		hu.setAverageMeanderLength(0.0d);
		hu.setSlope(.0011);
		return hu;
	}
	
	/**
	 * Setup channel width data
	 * @param rse a remote sensing estimator model
	 * @param hu a hydrological unit
	 */
	public void setupWidthData(RemoteSensingEstimator rse,HydrologicalUnit hu){
		Map<Integer,Double>data = new HashMap<Integer,Double>();
		for(int i=0; i < 12; i++){
			if(i<4 || i > 7)
				data.put(i,RandomHelper.nextDoubleFromTo(3, 8));
			else
				data.put(i,RandomHelper.nextDoubleFromTo(135, 145));
		}
		rse.getHydroloigcalWidth().put(hu, data);
	}
}
