package uaa.feral.social.demographics;

import repast.simphony.random.RandomHelper;
import uaa.feral.social.Community;
import uaa.feral.social.Person;
import junit.framework.TestCase;

/**
 * Class for testing demographic functions
 * @author 
 */
public class DemographicFunctionsTest extends TestCase{
	
	DemographicFunctions df;  //the demographic functions model
	Community community;  //the test community
	double[] rates={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};  //rates can be used for mortality
	private StandardDemographicTable sdt;  //demographic table contains probabilities based on age
	
	/**
	 * Setup the test
	 */
	public void setup(){
		community = new Community();
		sdt = new StandardDemographicTable(rates);
		for (int i=0 ; i < 100; i++){
			int age=RandomHelper.nextIntFromTo(0, 80);
			Sex sex = sdt.determineSex();
			Person p = new Person(null,age,sex,null);
			community.add(p);
		}
		
		df=new DemographicFunctions();
		df.setFertilityRate(.100);
		df.setMigrationRate(0);
		df.setCommunity(community);
		df.popRateCeiling=1.1;
		df.setTable(sdt);
	}
	
	/**
	 * Second setup for other tests
	 */
	public void setupTwo(){
		community = new Community();
		sdt = new StandardDemographicTable(rates);
		for (int i=0 ; i < 100; i++){
			int age=RandomHelper.nextIntFromTo(0, 80);
			Sex sex = sdt.determineSex();
			Person p = new Person(null,age,sex,null);
			community.add(p);
		}
		
		df=new DemographicFunctions();
		df.setFertilityRate(0);
		df.setMigrationRate(1.000);
		df.popRateCeiling=1.1;
		df.setCommunity(community);
		df.setTable(sdt);
	}
	
	/**
	 * Third setup for a third test
	 */
	public void setupThree(){
		community = new Community();
		sdt = new StandardDemographicTable(rates);
		for (int i=0 ; i < 100; i++){
			int age=RandomHelper.nextIntFromTo(0, 80);
			Sex sex = sdt.determineSex();
			Person p = new Person(null,age,sex,null);
			community.add(p);
		}
		
		df=new DemographicFunctions();
		df.setFertilityRate(0);
		df.setMigrationRate(-1.000);
		df.setCommunity(community);
		df.popRateCeiling=1.1;
		df.setTable(sdt);
	}
	
	/**
	 * Test birth
	 */
	public void testDemographicStep1(){
		setup();
		df.demographicSteps();
		assertTrue(df.birthMap.get(0).size()==10);
	}
	
	/**
	 * Test death
	 */
	public void testDemographicsStep2(){
		setup();
		for(int i=0; i < rates.length; i++){
			rates[i]=1.0;
		}
		sdt.setProbabilities(rates);
		df.setTable(sdt);
		df.demographicSteps();
		assertTrue(df.deathMap.get(0).size()==100);
	}
	
	/**
	 * Test immigration
	 */
	public void testDemographicStep3(){
		setupTwo();
		df.demographicSteps();
		assertTrue(df.birthMap.get(0).size()==100);
	}
	
	/**
	 * Test emigration
	 */
	public void testDemographicStep4(){
		setupThree();
		df.demographicSteps();
		assertTrue(df.deathMap.get(0).size()==100);
	}
}