package uaa.feral.social.waterUse;

import java.util.ArrayList;
import java.util.Iterator;

import repast.simphony.random.RandomHelper;
import uaa.feral.physical.HydrologicalSetting;
import uaa.feral.physical.HydrologicalUnit;
import uaa.feral.social.Community;
import uaa.feral.social.Person;
import uaa.feral.social.demographics.Sex;
import junit.framework.TestCase;

/**
 * Class to test function for social distancing water use model
 * @author 
 */
public class SocialDistancingTest extends TestCase{

	private Community community;  //community to test
	private HydrologicalUnit hydrologicalUnit;  //hydrological unit in water use model
	private SocialDistancing sd;  //social distancing water use model
	private double waterAmount;  //water amount of a given hydrology

	/**
	 * Setup method for test
	 */
	public void setup(){
		community = new Community();
		for (int i=0 ; i < 100; i++){
			Sex sex;
			int age=RandomHelper.nextIntFromTo(0, 80);
			double sexR=RandomHelper.nextDouble();
			if(sexR<=.5)
				sex=Sex.MALE;
			else
				sex=Sex.FEMALE;
			Person p = new Person(null,age,sex,null);
			community.add(p);
		}
		hydrologicalUnit=new HydrologicalUnit(null,null);
		hydrologicalUnit.setName("river");
		hydrologicalUnit.setIdentification(0);
		hydrologicalUnit.setDischarge(RandomHelper.nextDoubleFromTo(0, 200));
		this.waterAmount=hydrologicalUnit.getDischarge();
		HydrologicalSetting hs = new HydrologicalSetting("Test");
		hs.add(hydrologicalUnit);
		sd = new SocialDistancing(community);
		sd.waterSourceContexts=new ArrayList<HydrologicalUnit>();
		sd.initialValues();
		sd.determineSize(hs);
	}
	
	/**
	 * Test cehck person and quantity belief
	 */
	public void testCheckPerson(){
		setup();
		Iterator<Person> ic = community.getObjects(Person.class).iterator();
		while(ic.hasNext()){
			Person p = ic.next();
			sd.checkPerson(p);
			assertTrue(sd.quantityBelief.containsKey(p));
		}
	}
	
	/**
	 * Test select water source
	 */
	public void testSelectWaterSource(){
		setup();
		Iterator<Person> ic = community.getObjects(Person.class).iterator();
		sd.initializeWater();
		while(ic.hasNext()){
			Person p = ic.next();
			sd.checkPerson(p);
			HydrologicalUnit hu=sd.selectWaterSource(p);
			assertTrue(hydrologicalUnit==hu);
		}		
	}
	
	/**
	 * Test visit weighted preference
	 */
	public void testVisitWeightedPreference(){
		setup();
		Iterator<Person> ic = community.getObjects(Person.class).iterator();
		sd.initializeWater();
		while(ic.hasNext()){
			Person p = ic.next();
			sd.currentWaterSource.put(p, hydrologicalUnit);
			sd.checkPerson(p);
			sd.visitWeightedPreference(p);
			assertTrue(sd.sourcePreferences.get(p)[0]==1);
		}		
	}
	
	/**
	 * Test quantity history
	 */
	public void testQuantityHistory(){
		setup();
		Iterator<Person> ic = community.getObjects(Person.class).iterator();
		while(ic.hasNext()){
			Person p = ic.next();
			sd.checkPerson(p);
			sd.updateQuantityHistory(hydrologicalUnit, p, waterAmount);
			assertTrue(sd.quantityHistory.get(p).get(hydrologicalUnit)[0]==waterAmount);
		}
	}		
	
	/**
	 * Test quantity minimum
	 */
	public void testQuantityMinimum(){
		setup();
		Iterator<Person> ic = community.getObjects(Person.class).iterator();
		while(ic.hasNext()){
			Person p = ic.next();
			sd.checkPerson(p);
			sd.updateQuantityMinimum(hydrologicalUnit, p, waterAmount);
			assertTrue(sd.quantityMinimum.get(p).get(hydrologicalUnit)[0]==waterAmount);
		}		
	}
	
	/**
	 * Test quantity maximum
	 */
	public void testQuantityMaximum(){
		setup();
		Iterator<Person> ic = community.getObjects(Person.class).iterator();
		while(ic.hasNext()){
			Person p = ic.next();
			sd.checkPerson(p);
			sd.updateQuantityMaximum(hydrologicalUnit, p, waterAmount);
			assertTrue(sd.quantityMaximum.get(p).get(hydrologicalUnit)[0]==waterAmount);
		}		
	}
	
	/**
	 * Test quantity belief based on water amount
	 */
	public void testQuantityBelief(){
		setup();
		Iterator<Person> ic = community.getObjects(Person.class).iterator();
		while(ic.hasNext()){
			Person p = ic.next();
			sd.checkPerson(p);
			sd.updateQuantityHistory(hydrologicalUnit, p, waterAmount);
			sd.updateQuantityMaximum(hydrologicalUnit, p, waterAmount);
			sd.updateQuantityMinimum(hydrologicalUnit, p, waterAmount);
			sd.updateQuantityBelief(p,hydrologicalUnit);
			assertTrue(sd.quantityBelief.get(p)[0]==waterAmount);
		}		
	}
	
	/**
	 * Test quantity change belief value
	 */
	public void testQuantityChangeBelief(){
		setup();
		Iterator<Person> ic = community.getObjects(Person.class).iterator();
		while(ic.hasNext()){
			Person p = ic.next();
			sd.checkPerson(p);
			sd.updateQuantityChangeBelief(p, hydrologicalUnit);
			assertTrue(sd.quantityChangeBelief.get(p)[0]==.5);
		}		
	}
}
